#ifndef CMDGVISION_H
#define CMDGVISION_H
/** \file

   Definitions to handle data in various GammaVision formats.

*/
/*

   Copyright  2005-2009 M.Kraemer, GSI, D-64291 Darmstadt

   $Log: cmdgvision.h,v $
   Revision 1.1.1.1  2009/05/08 00:34:35  kraemer


   Revision 1.1.2.2  2009/05/08 00:14:41  kraemer
   64bit-clean, CE return

   Revision 1.1.2.1  2005/11/22 17:16:48  kraemer
   sync'ed with old CMD


*/

#include "cmderr.h"

                                      /* global flags */
#define CMDGVISION_DEBUG  0x80000000    /* general check flag */
#define CMDGVISION_QUIET  0x10000000    /* suppress error messages */
#define CMDGVISION_NOMSG  CMDGVISION_QUIET

#define CMDGVISION_MAXDIM 2

                                      /** \struct SCMDGVISIONWDW
                                          \brief GammaVision spectrum window.
                                      */ 
struct SCMDGVISIONWDW {
   int32_t lvLim[CMDGVISION_MAXDIM][4];  
};
                                      /* various input file formats */
enum { CMDGVISION_FMTNONE=0, CMDGVISION_FMTSPE, CMDGVISION_FMTTXT, };
                                      /** \struct SCMDGVISION
                                          \brief GammaVision data from file.
                                      */ 
struct SCMDGVISION {
   unsigned int uiFlag;               /**< hold general flag bits */
   unsigned int uiFmt;                /**< input file format */
   size_t ulSpace;                    /**< space occupied by tjps object */
   char *pcId;                        /**< spec ID */
   char **pvcRem;                     /**< [nRem] spec remarks */ 
   int  nRem;                         /**< # of spec remarks */
   char *pcFile;                      /**< file name */    
   char *pcDate;                      /**< acq start date */    
   char *pcTime;                      /**< acq start time */    
   char *pcUnit;                      /**< calibration units */

   int  nEff;                         /**< # of efficiency terms */
   double *pdEff;                     /**< [nEff] efficiency terms */     

   int  nMCA;                         /**< # of calibration terms */
   double *pdMCA;                     /**< [nMCA] MCA calibration terms */     

   int  nShape;                       /**< # of shape calibration terms */
   double *pdShape;                   /**< [nShape] shape calibration terms */     

   double dvEFit[3];                  /**< energy calibr. coeff. */
   double dvFFit[3];                  /**< FWHM calibr. coeff. ??? */
   double dvTime[2];                  /**< measuring time, [0/1] = real/live */ 
   int nWdw;                          /**< # of spectrum windows */
   struct SCMDGVISIONWDW *psWdw;      /**< [nWdw] window limits */ 
   int32_t  lvLim[CMDGVISION_MAXDIM][4]; /**< spectrum limits */
   ssize_t lSum;                      /**< spectrum sum (on read) */ 
   int32_t *plSpc;                    /**< spectrum data */ 
};

/*

   Functions
   ---------
   
   cmdGVisionFree    ( gvision, flag )
                     o releases resources
                   3 o to do
                     o calls:
                       o cmdGVisionInit()
                            
   cmdGVisionInit    ( gvision, flag )
                     o initialize
                   2 o to do
                     o calls:
                         
   cmdGVisionInspect ( gvision, flag )
                     o inspect 
                   1 o to do
                     o calls:
                         
   cmdGVisionRead    ( gvision, filename, flag )
                     o reads gammavision file
                   4 o to do
                     o calls:
                       o cmdGVisionInit     
                       o cmdGVisionFree     

   cmdGVisionWrite   ( gvision, filename, flag )
                     o writes gammavision file
                   5 o to do
                     o calls:
*/
                                      /* function prototypes */
struct SCMDERR cmdGVisionFree   ( struct SCMDGVISION *, unsigned int );
struct SCMDERR cmdGVisionInit   ( struct SCMDGVISION *, unsigned int );
struct SCMDERR cmdGVisionInspect( struct SCMDGVISION *, unsigned int );
struct SCMDERR cmdGVisionRead   ( struct SCMDGVISION *, const char *, unsigned int );

struct SCMDGVISION _cmdGVisionTest( unsigned int );

                                      /* error handling */
int cmdGVisionError( struct SCMDERR *, unsigned int );

enum { 
   ERRCMDGVISION_SYS=1,
   ERRCMDGVISION_BADADDR,
   ERRCMDGVISION_BADSYNTAX,
   ERRCMDGVISION_UNKNOWN,
   ERRCMDGVISION_BADFLOAT,
   ERRCMDGVISION_BADINTEGER,
   ERRCMDGVISION_BADTIME,
   ERRCMDGVISION_BADCHANNEL,
   ERRCMDGVISION_DATAEXTRANEOUS,
   ERRCMDGVISION_DATAMISSING,
   ERRCMDGVISION_BADNROI,
   ERRCMDGVISION_BADNCOEFF,
   ERRCMDGVISION_WDWMISSING,
};

#endif
