#ifndef CMDPRTCL_H
#define CMDPRTCL_H
/** \file
   Definitions for protocol stuff.
*/
/*
   
   Copyright  2002-2011 M.Kraemer, D-64291 Darmstadt, Seegartenstr. 32, 
                         all rights reserved

   $Log: cmdprtcl.h,v $
   Revision 1.2  2011/09/15 17:15:05  kraemer
   casts, types, constants

   Revision 1.1.1.1  2009/05/08 00:34:36  kraemer


   Revision 1.1.2.2  2009/05/08 00:17:16  kraemer
   64bit-clean, CE return

   Revision 1.1.2.1  2005/03/15 14:57:23  kraemer
   *** empty log message ***


*/

#if defined(__EMX__)
#define INCL_DOSSEMAPHORES
#include <os2.h>
#endif

#include "cmdsys.h"

                                      /** \struct SCMDPRTCL
                                          \brief Protocol stuff.
                                      */ 
struct SCMDPRTCL {
   unsigned int uiFlag;               /**< status flags */
   char *pcMsgFilter;                 /**< message filter string */
   int iFile;                         /**< protocol file handle */
   struct SCMDLOCK sCL;               /**< lock */
   char cFile[L_tmpnam+1L];            /**< protocol file name */
};
                                      /* status flags */ 
#define CMDPRTCL_ACTIVE 0x00000001    /* objects are setup  and active */        
#define CMDPRTCL_ON     0x00000002    /* switch protocol on/off */
                                      /* function specific flags */
#define CMDPRTCL_NONE    0x00000000
#define CMDPRTCL_TERM    0x00010000   /* output terminal */
#define CMDPRTCL_FILE    0x00020000   /* output file */
#define CMDPRTCL_NOTIME  0x00100000   /* don't include time stamp */
#define CMDPRTCL_NOCOM   0x00200000   /* don't preceed with comment char */

/*
  
   _cmdprtcl            ( cmdprtcl, str, flag )
                        o output text to terminal and/or protocol file 
                      6 o to test
                          o A
                          o O 
                          x X
                        o calls
                          o _cmdprtclSetup
  
   _cmdprtclCopy2File   ( cmdprtcl, file, flag )
                        o snapshot current protocol to file 
                      7 o to test
                          o A
                          o O 
                          x X
  
   _cmdprtclDelete      ( cmdprtcl, flag )
                        o delete current protocol (do not free resources)
                      8 o to test
                          o A
                          o O 
                          x X
  
   _cmdprtclFree        ( cmdprtcl, flag )
                        o release resources 
                      3 o to test
                          o A
                          o O 
                          x X
                        o problem: shutdown when other tasks hold the lock
                        o calls:
                          o _cmdprtclInit()
  
  
   _cmdprtclImportMsgFilter( cmdprtcl, str, flag )
                        o set message filter 
                      5 o to test
                          o A
                          o O 
                          x X
                        o calls:
  
   _cmdprtclInit        ( cmdprtcl, flag )
                        o initialize to "zero" state 
                      2 o to test
                          o A
                          o O 
                          x X
  
   _cmdprtclInspect     ( cmdprtcl, flag )
                        o inspect  
                      1 o to test
                          o A
                          o O 
                          x X
  
   _cmdprtclOn/Off      ( cmdprtcl, flag )
                        o switch protocol on/off 
                      9 o to test
                          o A
                          o O 
                          x X
  
   _cmdprtclSetup       ( cmdprtcl, flag )
                        o initialize for use 
                      4 o to test
                          o A
                          o O 
                          x X
                        o calls:
                          o _cmdprtclInit()
                          o _cmdprtclFree()
  
*/
                                      /* function prototypes */
int  _cmdprtcl         ( struct SCMDPRTCL *, const char *, unsigned int );
int  _cmdprtclCopy2File( struct SCMDPRTCL *, const char *, unsigned int );
int  _cmdprtclDelete   ( struct SCMDPRTCL *, unsigned int );
int  _cmdprtclFree     ( struct SCMDPRTCL *, unsigned int );
int  _cmdprtclImportMsgFilter( struct SCMDPRTCL *, const char *, unsigned int );
int  _cmdprtclInit     ( struct SCMDPRTCL *, unsigned int );
int  _cmdprtclInspect  ( struct SCMDPRTCL *, unsigned int );
int  _cmdprtclOn       ( struct SCMDPRTCL *, unsigned int );
int  _cmdprtclOff      ( struct SCMDPRTCL *, unsigned int );
int  _cmdprtclSetup    ( struct SCMDPRTCL *, unsigned int );
int  _cmdprtclTest     ( struct SCMDPRTCL *, unsigned int );
void _prtcl            (       const char *, unsigned int );

#endif
