#ifndef UMMAP_H
#define UMMAP_H

/** \file Definitions for file/memory mapping services.
*/

/*
   Copyright  2000-2009 M.Kraemer, D-64291 Darmstadt, Seegartenstr. 32

   $Log: ummap.h,v $
   Revision 1.2  2011/09/15 16:53:43  kraemer
   special mmap for HP-UX

   Revision 1.1.1.1  2009/05/06 16:21:41  kraemer


   Revision 1.1.2.5  2009/05/06 14:09:37  kraemer
   64bit-clean

   Revision 1.1.2.4  2005/03/12 00:45:20  kraemer
   *** empty log message ***

   Revision 1.1.2.3  2005/03/11 19:07:21  kraemer
   AIX sync

   Revision 1.1.2.2  2005/03/11 18:31:54  kraemer
   MacOS X sync

   Revision 1.1.2.1  2005/01/31 20:40:39  kraemer
   *** empty log message ***


*/

#include <sys/types.h>
#include <sys/stat.h>

#if defined(AMIGA)
 
#include <dos.h>
#include <clib/dos_protos.h>
#include <proto/exec.h>
#include <exec/memory.h>

#elif defined(__EMX__)

#define INCL_PM
#define INCL_BASE
#include <os2.h>

#undef UMMAP_TRUE

#if defined(UMMAP_TRUE) 
#include "mman.h"
#endif

#else

#include <sys/mman.h>


#include <fcntl.h>

#endif

#if !defined(MAP_FILE)
#define MAP_FILE 0
#endif
#if !defined(MAP_FAILED)
#define MAP_FAILED ((void *) -1)
#endif

#define UMMAP_MASKFUNC 0x00FF0000     /* function specific flags */

#define UMMAP_MASK    0xFF000000
#define UMMAP_ERROR   0x80000000
#define UMMAP_WARNING 0x40000000

#define UMMAP_DEBUG  0x80000000       /* general check flag */
#define UMMAP_QUIET  0x10000000       /* suppress error messages */
#define UMMAP_NOMSG  UMMAP_QUIET
                                      /** \struct SUMMAP
                                          \brief Describes file-memory mapping.
                                      */ 
struct SUMMAP {
   unsigned int uiFlag;               /**< holds flag bits */
#define UMMAP_ALLOC  0x00000001       /* pseudo-map: memory allocated */ 
#define UMMAP_LOCKED 0x00000002       /* file has been locked */
#define UMMAP_REFMAP 0x00000010       /* map is reference only (e.g. on HP-UX), don't unmap */
#if defined(AMIGA)
   BPTR fl;                           /**< file lock */     
   BPTR fh;                           /**< file handle */
   __aligned struct FileInfoBlock sFIB;
#elif defined(__EMX__)
#if defined(UMMAP_TRUE)
   mmap_reg_t mmap_reg;               /**<  */
#endif
   FILELOCK fl;                       /**<  */
   HFILE fh;                          /**<  */
   FILESTATUS3 fs3;                   /**<  */
#endif
   off_t offFileSize;                 /**< filesize (32/64 bit) in bytes */      
   int iFile;                         /**< file descriptor for memory mapping */ 
   char *pcFileName;                  /**< copy of file name */    
   void *pv;                          /**< ptr. to mapped memory */
   struct stat sStat;                 /**< file stat */ 
};
                                      /* flag bits */
 
                                      /* file open modes */
#define UMMAP_READ   0x00000001       /* file open for read */
#define UMMAP_WRITE  0x00000002       /* file open for write */
#define UMMAP_APPEND 0x00000004       /* file open for append */

                                      /* function action flags */
#define UMMAPCLOSE_DELETE   0x00010000/* delete file on close */ 
#define UMMAPCLOSE_FILEONLY 0x00020000/* special for fake maps: close file, but keep memory */ 

                                      /* aux. macros */ 
#define UMMAPISALLOC(m)  ( (m)->uiFlag & UMMAP_ALLOC )
#define UMMAPISLOCKED(m) ( (m)->uiFlag & UMMAP_LOCKED )
 
/*
   General:
   --------

   o OS/2 branches:
     o http://www.developer.ibm.com/library/p_guides.html

   o support >4GB too

   Macros
   ------

   UMMAPISALLOC (&(psFM -> sMM))
   
   UMMAPISLOCKED(&(psFM -> sMM))

   Functions
   ---------

   uMMapClose         ( mmap, flag )
                      o close file
                    4 x to complete/test
                        x X  
                        o A
                        o O 
                      o honour UMMAPCLOSE_DELETE
                        x test
                        o use file handle rather than file name  
                      o calls:
                        o uMMapInit
                        o uMMapSync

   uMMapInit          ( mmap, flag )
                      o initialize
                    2 x to test
                      o calls:

   uMMapInspect       ( mmap, flag ) 
                      o inspect output
                    1 x to test
                      o calls:

   uMMapIsFileMapped  ( mmap, file, flag  )
                      o check if file is mapped, returns 0 for mapped, !=0 else
                    7 x to test
                      o calls
                        o fileeq()  

   uMMapIsValid       ( mmap, flag ) 
                      o check valid memory
                    6 x to test
                      o calls:

   uMMapOpen          ( mmap, file, size, mode, flag )
                      o open for read/write (provides memory block address) 
                    5 . to complete/test
                        ? write only vs read/write
                        ? if file exists, shorter maps do not truncate, OK ?
                        o append
                        o write access w/o creat/trunc,
                          allow multiple writes from different tasks/threads/processes
                        o X:                     
                          o lock if possible
                        o O: 
                          x test if lock necessary => no
                          o true mmap
                            EALib error 5 in DosSetFileInfo
                            EALib error 5 in DosSetFileInfo
                        o A
                          o Lock() for all modes:
                            use packets with file-handle ?
                          o true mmap   
                      o calls:
                        o uMMapInit()
                        o uMMapClose()

   uMMapSync          ( mmap, flag )
                      o flush memory buffer
                    3 o to complete/test
                        o X N/A
                        o A
                        o O 
                      o calls:


*/
                                      /* function prototypes */
int uMMapClose       ( struct SUMMAP *, unsigned int );
int uMMapInit        ( struct SUMMAP *, unsigned int );
int uMMapInspect     ( struct SUMMAP *, unsigned int );
int uMMapIsFileMapped( struct SUMMAP *, const char *, unsigned int );
int uMMapIsValid     ( struct SUMMAP *, unsigned int );
int uMMapOpen        ( struct SUMMAP *, const char *, off_t, unsigned int, unsigned int );
int uMMapSync        ( struct SUMMAP *, unsigned int );

struct SUMMAP *_uMMapTest( unsigned int );

/*
   --- error codes
*/

enum { 
   ERRUMMAP_SYS=1, 
   ERRUMMAP_BADADDR,
   ERRUMMAP_BADFILE,
   ERRUMMAP_BADMEMORY,
   ERRUMMAP_FILENOTMAPPED,
   ERRUMMAP_BADFILESIZE, 
   ERRUMMAP_BADFILENAME, 
   ERRUMMAP_FILENOTREGULAR,
   ERRUMMAP_DOSOPENFAILED,
   ERRUMMAP_DOSALLOCMEMFAILED,
   ERRUMMAP_DOSREADFAILED, 
   ERRUMMAP_DOSWRITEFAILED, 
   ERRUMMAP_DOSQUERYFILEINFOFAILED,
   ERRUMMAP_LOCKFAILED,
   ERRUMMAP_EXAMINEFAILED, 
   ERRUMMAP_LARGEFILE,
};

#endif
